/**
 *
 *  Copyright Openmind http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlcriteria.jcr.query;

import net.sourceforge.openutils.mgnlcriteria.jcr.query.xpath.impl.MagnoliaCriteriaImpl;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.xpath.impl.MagnoliaCriteriaWithLimitImpl;


/**
 * @author fgrilli
 * @version $Id: JCRCriteriaFactory.java 1193 2009-05-08 21:05:22Z fgrilli $
 */
public class JCRCriteriaFactory
{

    private JCRCriteriaFactory()
    {
    }

    /**
     * A factory for Magnolia CMS specific Criteria implementation
     * @param path String - the path in JCR repository to search for Nodes. This is the path which precedes the
     * predicate
     * @param queryManager - an instance of info.magnolia.cms.core.search.QueryManager
     * @param itemType String - if not specified defaults to "mgnl:content"
     * @return a factory for Magnolia CMS specific Criteria implementation
     */
    public static Criteria createMgnlCriteria(String path, info.magnolia.cms.core.search.QueryManager queryManager,
        String itemType)
    {
        return new MagnoliaCriteriaImpl(path, queryManager, itemType);
    }

    /**
     * A factory for Magnolia CMS specific Criteria implementation
     * @param path String - the path in JCR repository to search for Nodes. This is the path which precedes the
     * predicate
     * @param queryManager - an instance of info.magnolia.cms.core.search.QueryManager
     * @param itemType String - if not specified defaults to "mgnl:content"
     * @param classType Class<?> - the list method of this implementation must return instances of this type
     * @return a factory for Magnolia CMS specific Criteria implementation
     */
    public static Criteria createMgnlCriteria(String path, info.magnolia.cms.core.search.QueryManager queryManager,
        String itemType, Class< ? > classType)
    {
        return new MagnoliaCriteriaImpl(path, queryManager, itemType, classType);
    }

    /**
     * A factory for Magnolia CMS specific Criteria implementation which can return a subset of results instead of the
     * whole resultset, much like a MySQL limit clause. For this to work, the underlying JCR repository implementation
     * must support this feature (Jackrabbit 1.4+ does).<br>
     * To set the maximum number of returned items you must call {@link Criteria#setMaxResults(int)} (by default
     * {@link Criteria#setFirstResult(int)} is set to 0).<br>
     * Please notice that if {@link Criteria#setMaxResults(int)} is not explicitly called before the
     * {@link Criteria#list()} method, the latter will return the entire resultset as in
     * {@link JCRCriteriaFactory#createMgnlCriteria(String, info.magnolia.cms.core.search.QueryManager, String)}
     * @param path String - the path in JCR repository to search for Nodes. This is the path which precedes the
     * predicate
     * @param queryManager - an instance of info.magnolia.cms.core.search.QueryManager
     * @param itemType String - if not specified defaults to "mgnl:content"
     * @return a factory for Magnolia CMS specific Criteria implementation
     */
    public static Criteria createMgnlCriteriaWithLimit(String path,
        info.magnolia.cms.core.search.QueryManager queryManager, String itemType)
    {
        return new MagnoliaCriteriaWithLimitImpl(path, queryManager, itemType);
    }

    /**
     * A factory for Magnolia CMS specific Criteria implementation which can return a subset of results instead of the
     * whole resultset, much like a MySQL limit clause. For this to work, the underlying JCR repository implementation
     * must support this feature (Jackrabbit 1.4+ does).<br>
     * To set the maximum number of returned items you must call {@link Criteria#setMaxResults(int)} (by default
     * {@link Criteria#setFirstResult(int)} is set to 0).<br>
     * Please notice that if {@link Criteria#setMaxResults(int)} is not explicitly called before the
     * {@link Criteria#list()} method, the latter will return the entire resultset as in
     * {@link JCRCriteriaFactory#createMgnlCriteria(String, info.magnolia.cms.core.search.QueryManager, String, Class)}
     * @param path String - the path in JCR repository to search for Nodes. This is the path which precedes the
     * predicate
     * @param queryManager - an instance of info.magnolia.cms.core.search.QueryManager
     * @param itemType String - if not specified defaults to "mgnl:content"
     * @param classType Class<?> - the list method of this implementation must return instances of this type
     * @return a factory for Magnolia CMS specific Criteria implementation
     */
    public static Criteria createMgnlCriteriaWithLimit(String path,
        info.magnolia.cms.core.search.QueryManager queryManager, String itemType, Class< ? > classType)
    {
        return new MagnoliaCriteriaWithLimitImpl(path, queryManager, itemType, classType);
    }

    /*
     * @param path String - the path in JCR repository to search for Nodes. This is the path which precedes the
     * predicate
     * @param queryManager - an instance of javax.jcr.query.QueryManager
     * @param itemType String - depends on the implementation
     * @return an instance of a JCR-specific implementation of {@link Criteria}
     * @throws UnsupportedOperationException as it is not implemented yet public static Criteria
     * createJCRCriteria(String path, javax.jcr.query.QueryManager queryManager, String itemType) { throw new
     * UnsupportedOperationException(); }
     */
}
