/**
 *
 *  Copyright Openmind http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlcriteria.jcr.query.criterion;


import java.io.Serializable;

import net.sourceforge.openutils.mgnlcriteria.jcr.query.Criteria;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.JCRQueryException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * An object-oriented representation of a query criterion that may be used as a restriction in a <tt>Criteria</tt>
 * query. Built-in criterion types are provided by the <tt>Restrictions</tt> factory class. This interface might be
 * implemented by application classes that define custom restriction criteria.
 * @see Restrictions
 * @see Criteria
 * @author Federico Grilli
 * @version $Id: Criterion.java 1193 2009-05-08 21:05:22Z fgrilli $
 */
public interface Criterion extends Serializable
{

    public static final Logger log = LoggerFactory.getLogger(Criterion.class);

    public static final String ATTRIBUTE_SELECTOR = "@";

    public static final String JCR_PREFIX = "jcr:";

    public static final String JCR_ROOT = "/jcr:root";

    public static final String ALL_ELEMENTS = "//*";

    public static final String ANY_PROPERTY = ".";

    public static final String XS_DATETIME_FUNCTION = "xs:dateTime";

    public static final String MIDNIGHT = "T00:00:00.000+00:00";

    public static final String T235959 = "T23:59:59.000+00:00";

    public static final String NT_BASE = "nt:base";

    public static final String NOT = " not(";

    public static final String HYPHEN = "-";

    public static final int DAY_MAX = 10;

    public static final int MONTH_MAX = 9;

    /**
     * Render the XPath fragment
     * @param criteria
     * @return String
     * @throws JCRQueryException
     */
    public String toXPathString(Criteria criteria) throws JCRQueryException;

}
