/**
 *
 *  Copyright Openmind http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlcriteria.jcr.query.criterion;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.sourceforge.openutils.mgnlcriteria.jcr.query.Criteria;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.JCRQueryException;

import org.apache.commons.lang.StringUtils;


/**
 * A sequence of a logical expressions combined by some associative logical operator
 * @author Federico Grilli
 * @version $Id: Junction.java 1193 2009-05-08 21:05:22Z fgrilli $
 */
public class Junction implements Criterion
{

    private static final long serialVersionUID = 4745761472724863693L;

    @SuppressWarnings("unchecked")
    private final List criteria = new ArrayList();

    private final String op;

    protected Junction(String op)
    {
        this.op = op;
    }

    @SuppressWarnings("unchecked")
    public Junction add(Criterion criterion)
    {
        criteria.add(criterion);
        return this;
    }

    public String getOp()
    {
        return op;
    }

    @SuppressWarnings("unchecked")
    public String toXPathString(Criteria crit) throws JCRQueryException
    {

        if (criteria.size() == 0)
            return "";

        StringBuilder buffer = new StringBuilder().append('(');
        Iterator iter = criteria.iterator();
        while (iter.hasNext())
        {
            buffer.append(((Criterion) iter.next()).toXPathString(crit));
            if (iter.hasNext())
                buffer.append(' ').append(op).append(' ');
        }
        return buffer.append(')').toString();
    }

    /**
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString()
    {
        return '(' + StringUtils.join(criteria.iterator(), ' ' + op + ' ') + ')';
    }

    /**
     * @return true if this Criterion contains at least one expression, false otherwise
     */
    public boolean isEmpty()
    {
        return criteria.isEmpty();
    }
}
