/**
 *
 *  Copyright Openmind http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlcriteria.jcr.query.criterion;


import java.util.Calendar;

import net.sourceforge.openutils.mgnlcriteria.jcr.query.Criteria;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.JCRQueryException;


/**
 * superclass for "simple" comparisons (with XPATH binary operators)
 * @author Federico Grilli
 * @version $Id: SimpleExpression.java 1193 2009-05-08 21:05:22Z fgrilli $
 */
public class SimpleExpression implements Criterion
{

    private static final long serialVersionUID = -1104419394978535803L;

    private final String propertyName;

    private final Object value;

    private final String op;

    protected SimpleExpression(String propertyName, Object value, String op)
    {
        this.propertyName = propertyName;
        this.value = value;
        this.op = op;
    }

    @Override
    public String toString()
    {
        return propertyName + getOp() + value;
    }

    protected final String getOp()
    {
        return op;
    }

    public String toXPathString(Criteria criteria) throws JCRQueryException
    {
        StringBuilder fragment = new StringBuilder();
        fragment.append(" (").append(propertyName).append(getOp());

        if (value instanceof String)
            fragment.append("'" + value + "') ");
        else if (value instanceof Number)
            fragment.append(value + ") ");
        else if (value instanceof Calendar)
        {
            Calendar cal = (Calendar) value;
            if ("=".equals(op))
            {
                // trick to make it work with dates
                return Restrictions.between(propertyName, value, value).toXPathString(null);

            }
            else
            {
                StringBuilder date = new StringBuilder();
                date.append(cal.get(Calendar.YEAR)).append(HYPHEN).append(
                    cal.get(Calendar.MONTH) < MONTH_MAX
                        ? "0" + (cal.get(Calendar.MONTH) + 1)
                        : cal.get(Calendar.MONTH) + 1).append(HYPHEN).append(
                    cal.get(Calendar.DAY_OF_MONTH) < DAY_MAX ? "0" + cal.get(Calendar.DAY_OF_MONTH) : cal
                        .get(Calendar.DAY_OF_MONTH)).append(MIDNIGHT);
                fragment.append(XS_DATETIME_FUNCTION + "('" + date.toString() + "')) ");
            }
        }
        else
        {
            String msg = "value is not one of the accepted types String, Number, Calendar";
            log.error(msg);
            throw new IllegalArgumentException(msg);
        }
        log.debug("xpathString is {} ", fragment);
        return fragment.toString();
    }
}
