/**
 *
 *  Copyright Openmind http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlcriteria.jcr.query.xpath.impl;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.search.Query;
import info.magnolia.cms.core.search.QueryManager;
import info.magnolia.cms.core.search.QueryResult;
import info.magnolia.content2bean.Content2BeanException;
import info.magnolia.content2bean.Content2BeanUtil;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.jcr.RepositoryException;
import javax.jcr.query.InvalidQueryException;

import net.sourceforge.openutils.mgnlcriteria.jcr.query.JCRMagnoliaCriteriaQueryTranslator;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.JCRQueryException;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.xpath.XPathSelect;


/**
 * Magnolia XPATH implementation of the <tt>Criteria</tt> interface
 * @author Federico Grilli
 * @version $Id: MagnoliaCriteriaImpl.java 1193 2009-05-08 21:05:22Z fgrilli $
 */
public class MagnoliaCriteriaImpl extends AbstractMagnoliaCriteriaImpl
{

    protected static final String EXCEPTION_MSG = "An error occurred while executing a query. Xpath query was {}. Exception message is {}";

    public MagnoliaCriteriaImpl(String path, QueryManager queryManager, String itemType)
    {
        super(path, queryManager, itemType, null);
    }

    public MagnoliaCriteriaImpl(String path, QueryManager queryManager, String itemType, Class< ? > classType)
    {
        super(path, queryManager, itemType, classType);
    }

    @SuppressWarnings("unchecked")
    @Override
    public Collection< ? > list() throws JCRQueryException
    {
        if (queryManager == null)
            throw new IllegalStateException("QueryManager cannot be null");

        log.debug("creating statement...");
        long start = System.currentTimeMillis();

        JCRMagnoliaCriteriaQueryTranslator translator = new JCRMagnoliaCriteriaQueryTranslator(this);
        XPathSelect statement = new XPathSelect();
        statement.setRoot(this.path);
        statement.setPredicate(translator.getPredicate());
        statement.setOrderByClause(translator.getOrderBy());
        String stmt = statement.toStatementString();

        long stop = System.currentTimeMillis();
        log.debug("statement created in {} ms", stop - start);
        Collection< ? > retVal;
        try
        {

            Query q = queryManager.createQuery(stmt, Query.XPATH);

            log.debug("executing Query {}", q.getStatement());
            start = System.currentTimeMillis();

            QueryResult r = q.execute();

            stop = System.currentTimeMillis();
            log.debug("Query executed in {} ms", stop - start);

            log.debug("getting content...");
            start = System.currentTimeMillis();

            retVal = r.getContent(itemType);

            stop = System.currentTimeMillis();

            if (Content.class.isAssignableFrom(classType))
            {
                log.debug("returning a Collection holding objects of type {} ", Content.class.getName());
                retVal = retVal == null ? new ArrayList<Content>() : retVal;
                log.debug("got {} node(s) in {} ms", new Object[]{retVal.size(), stop - start });

                return retVal;

            }
            else
            {
                log.debug("returning a Collection holding objects of type {} ", classType.getName());
                Iterator iter = retVal.iterator();
                List list = new ArrayList();
                while (iter.hasNext())
                {
                    Content node = (Content) iter.next();
                    try
                    {
                        list.add(classType.cast(Content2BeanUtil.toBean(node, true, classType)));
                    }
                    catch (Content2BeanException e)
                    {
                        log.error(e.getMessage());
                        throw new JCRQueryException(e);
                    }
                }
                log.debug("got {} node(s) in {} ms", new Object[]{list.size(), stop - start });
                return list;
            }

        }
        catch (InvalidQueryException e)
        {
            log.error(EXCEPTION_MSG, new Object[]{stmt, e.getMessage() });
            throw new JCRQueryException(e);
        }
        catch (RepositoryException e)
        {
            log.error(EXCEPTION_MSG, new Object[]{stmt, e.getMessage() });
            throw new JCRQueryException(e);
        }
    }
}